--[[

Zotwee price source for TradeSkillMaster addon, v1.0.4
http://zotwee.info/

** This is NOT an official TradeSkillMaster addon **

]]

local ZWM = select(2, ...)
ZWM = LibStub("AceAddon-3.0"):NewAddon(ZWM, "TSM_Zotwee", "AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("TradeSkillMaster_Zotwee") -- loads the localization table
ZWM.version = "1.0.4"
ZWM.moduleName = "TradeSkillMaster Zotwee"

local savedDBDefaults = {
  realm = {},
  profile = {
    tooltip = true,
    marketValue = true,
    last24Price = true,
    last30Price = true,
    lastSeen = true,
    minBuyout = true,
    quantity = true,
    suggestion = true,
    suggestionSource = 1
  }
}
local WHITE  = "|cFFFFFFFF"
local YELLOW = "|cFFFFFF00"
local GREEN  = "|cFF00FF00"
local RED    = "|cFFFF0000"
local CLEAR  = "|r"
-- Called once the player has loaded WOW.
function ZWM:OnInitialize()
  ZWM.db = LibStub:GetLibrary("AceDB-3.0"):New("TradeSkillMaster_ZotweeDB", savedDBDefaults, true)

  Zotwee:AddModule(ZWM)
  -- register this module with TSM
  ZWM:RegisterModule()
end

-- registers this module with TSM by first setting all fields and then calling TSMAPI:NewModule().
function ZWM:RegisterModule()
  ZWM.priceSources = {
    { key = "zotweeMarket", label = L["Zotwee Realm Market Value"], callback = "GetMarketData", arg = "market" },
    { key = "zotwee24h", label = L["Zotwee Last 24 hours Price"], callback = "GetMarketData", arg = "market24h" },
    { key = "zotwee30d", label = L["Zotwee Last 30 days Price"], callback = "GetMarketData", arg = "market30d" }
  }
  ZWM.tooltipOptions = { callback = "LoadTooltipOptions" }

  TSMAPI:NewModule(ZWM)
end

function ZWM:GetMarketData(itemID, key)
  if type(itemID) ~= "number" then itemID = TSMAPI:GetItemID(itemID) end

  local data = ZotweeMarketItem(itemID)

  if not data then return end

  if not key then return data end

  if data[key] and data[key] > 0 then
    return data[key]
  else
    return
  end
end

local tooltip
local function isSoulbound(itemid)
  if not tooltip then
    tooltip = CreateFrame("GameTooltip", "TSMZotweeTooltip", nil, "GameTooltipTemplate")
    tooltip:SetOwner(WorldFrame, "ANCHOR_NONE")
  end
  tooltip:ClearLines()
  
  local link = tostring(itemid)
  if not link:match("item:") then
    link = "item:"..link
  end
  tooltip:SetHyperlink(link)

  for i=2, tooltip:NumLines() do
    local left = _G["TSMZotweeTooltipTextLeft"..i]
    if left and left:IsShown() then
      if string.match(left:GetText(), ITEM_SOULBOUND) or string.match(left:GetText(), ITEM_BIND_ON_PICKUP) or string.match(left:GetText(), ITEM_BIND_TO_BNETACCOUNT) then return true end
    end
  end
 
  return false
end

function ZWM:GetTooltip(itemString, quantity)
  if not ZWM.db.profile.tooltip then return end
  if not strfind(itemString, "item:") then return end
  local itemID = TSMAPI:GetItemID(itemString)
  if not itemID then return end
  local data = ZWM:GetMarketData(itemString)
  local moneyCoinsTooltip = TSMAPI:GetMoneyCoinsTooltip()
  local text = {}

  if data then

    if ZWM.db.profile.marketValue then
      if data.market and data.market > 0 then
        if moneyCoinsTooltip then
          if IsShiftKeyDown() then
            tinsert(text, { left = "  " .. format(L["Realm Market x%s:"], quantity), right = TSMAPI:FormatTextMoneyIcon(data.market * quantity, "|cffffffff", true) })
          else
            tinsert(text, { left = "  " .. L["Realm Market:"], right = TSMAPI:FormatTextMoneyIcon(data.market, "|cffffffff", true) })
          end
        else
          if IsShiftKeyDown() then
            tinsert(text, { left = "  " .. format(L["Realm Market x%s:"], quantity), right = TSMAPI:FormatTextMoney(data.market * quantity, "|cffffffff", true) })
          else
            tinsert(text, { left = "  " .. L["Realm Market:"], right = TSMAPI:FormatTextMoney(data.market, "|cffffffff", true) })
          end
        end
      end
    end
    if ZWM.db.profile.last24Price then
      if data.market24h and data.market24h > 0 then
        if moneyCoinsTooltip then
          if IsShiftKeyDown() then
            tinsert(text, { left = "  " .. format(L["Realm 24hrs Market x%s:"], quantity), right = TSMAPI:FormatTextMoneyIcon(data.market24h * quantity, "|cffffffff", true) })
          else
            tinsert(text, { left = "  " .. L["Realm 24hrs Market:"], right = TSMAPI:FormatTextMoneyIcon(data.market24h, "|cffffffff", true) })
          end
        else
          if IsShiftKeyDown() then
            tinsert(text, { left = "  " .. format(L["Realm 24hrs Market x%s:"], quantity), right = TSMAPI:FormatTextMoney(data.market24h * quantity, "|cffffffff", true) })
          else
            tinsert(text, { left = "  " .. L["Realm 24hrs Market:"], right = TSMAPI:FormatTextMoney(data.market24h, "|cffffffff", true) })
          end
        end
      end
    end
    if ZWM.db.profile.last30Price then
      if data.market30d and data.market30d > 0 then
        if moneyCoinsTooltip then
          if IsShiftKeyDown() then
            tinsert(text, { left = "  " .. format(L["Realm 30days Market x%s:"], quantity), right = TSMAPI:FormatTextMoneyIcon(data.market30d * quantity, "|cffffffff", true) })
          else
            tinsert(text, { left = "  " .. L["Realm 30days Market:"], right = TSMAPI:FormatTextMoneyIcon(data.market30d, "|cffffffff", true) })
          end
        else
          if IsShiftKeyDown() then
            tinsert(text, { left = "  " .. format(L["Realm 30days Market x%s:"], quantity), right = TSMAPI:FormatTextMoney(data.market30d * quantity, "|cffffffff", true) })
          else
            tinsert(text, { left = "  " .. L["Realm 30days Market:"], right = TSMAPI:FormatTextMoney(data.market30d, "|cffffffff", true) })
          end
        end
      end
    end
    if ZWM.db.profile.minBuyout then
      if data.minbuyout and data.minbuyout > 0 then
        if moneyCoinsTooltip then
          tinsert(text, { left = "  " .. L["Realm Min Buyout:"], right = TSMAPI:FormatTextMoneyIcon(data.minbuyout, "|cffffffff", true) })
        else
          tinsert(text, { left = "  " .. L["Realm Min Buyout:"], right = TSMAPI:FormatTextMoney(data.minbuyout, "|cffffffff", true) })
        end
      end
    end
  
    if ZWM.db.profile.lastSeen then
        if data.age then
          tinsert(text, { left = "  " .. L["Last Seen:"], right = "|cffffffff" .. format(L["%s ago"], SecondsToTime(data.age)) })
        end
    end
  
    if ZWM.db.profile.quantity then
        if data.qtd then
          tinsert(text, { left = "  " .. L["Quantity:"], right = "|cffffffff" .. data.qtd })
        end
    end
  end
    
  if ZWM.db.profile.suggestion then
    local suggestion = L["Do nothing"]
    local deValue = ZWM:GetDisenchantValue(itemString)
    local millValue = ZWM:GetMillValue(itemString)
    local prospectValue = ZWM:GetProspectValue(itemString)
    local vendorValue = select(11, TSMAPI:GetSafeItemInfo(itemString))
    local winnerPrice = 0
      
	  tinsert(text, {left = WHITE.."  Zotwee "..L["Suggestion"]..CLEAR})
	  
	  if vendorValue and vendorValue > 0 then
	    winnerPrice = vendorValue * 0.95
		suggestion = L["Vendor"]
		tinsert(text, {left = "  "..L["Vendor"]..": "..CLEAR, right = TSMAPI:FormatTextMoney(vendorValue, "|cffffffff", true)})
	  end
      -- Can't auction a sould bound item
      local bind = isSoulbound(itemid)	  
      if not bind then
        local auctionPrice
        if ZWM.db.profile.suggestionSource == 2 then
          if data and data["market"] then
            auctionPrice = data["market"]
          end
        elseif ZWM.db.profile.suggestionSource == 1 and IsAddOnLoaded("TradeSkillMaster_AuctionDB") then
          auctionPrice = TSMAPI:GetItemValue(itemString, "DBMarket")
        end
        
        if auctionPrice and auctionPrice > 0 then
          tinsert(text, {left = "  "..L["Auction"]..": "..CLEAR, right = TSMAPI:FormatTextMoney(auctionPrice, "|cffffffff", true)})
          if auctionPrice > winnerPrice then
            winnerPrice = auctionPrice
            suggestion = L["Auction"]
          end
        end
      end

      if deValue and deValue > 0 then
        if deValue > winnerPrice then
          winnerPrice = deValue
          suggestion = L["Disenchant"]
        end
        tinsert(text, {left = "  "..L["Disenchant"]..": "..CLEAR, right = TSMAPI:FormatTextMoney(deValue, "|cffffffff", true)})
      end
      
      if millValue and millValue > 0 then
        if millValue > winnerPrice then
          winnerPrice = millValue
          suggestion = L["Mill"]
        end
        tinsert(text, {left = "  "..L["Mill"]..": "..CLEAR, right = TSMAPI:FormatTextMoney(millValue, "|cffffffff", true)})
      end
      
      if prospectValue and prospectValue > 0 then
        if prospectValue > winnerPrice then
          winnerPrice = prospectValue
          suggestion = L["Prospect"]
        end
        tinsert(text, {left = "  "..L["Prospect"]..": "..CLEAR, right = TSMAPI:FormatTextMoney(prospectValue, "|cffffffff", true)})
      end
      
      if winnerPrice then
        tinsert(text, {left = "  "..L["Suggestion"]..":", right = GREEN..suggestion..CLEAR})
      end
  end
      
  if #text > 0 then
    local lastScan = Zotwee:GetLastUpdateTime()
    if lastScan then
      local diff = SecondsToTime(time() - lastScan)
      tinsert(text, 1, { left = "|cffffff00" .. "TSM Zotwee:", right = "|cffffffff" .. format(L["%s ago"], diff) })
    else
      tinsert(text, 1, { left = "|cffffff00" .. "TSM Zotwee:", right = "|cffffffff" .. L["No data yet"] })
    end
    return text
  end
end

function ZWM:GetDisenchantValue(itemString)
	local itemID = TSMAPI:GetItemID(itemString)
	local name, link, quality, ilvl, reqLevel, iType, subclass, maxStack, equipSlot, texture, vendorPrice = GetItemInfo(itemID)	
	local WEAPON, ARMOR = GetAuctionItemClasses()

	if not itemString or TSMAPI.DisenchantingData.notDisenchantable[itemString] or not (iType == ARMOR or iType == WEAPON) then return end

	local value = 0
	for _, data in ipairs(TSMAPI.DisenchantingData.disenchant) do
		for item, itemData in pairs(data) do
			if item ~= "desc" and itemData.itemTypes[iType] and itemData.itemTypes[iType][quality] then
				for _, deData in ipairs(itemData.itemTypes[iType][quality]) do
					if ilvl >= deData.minItemLevel and ilvl <= deData.maxItemLevel then
						local matValue = TSMAPI:GetItemValue(item, "DBMarket")
						value = value + (matValue or 0) * deData.amountOfMats
					end
				end
			end
		end
	end

	value = floor(value)
	return value > 0 and value or nil
end

function ZWM:GetMillValue(itemString)
	local value = 0
	
	for _, targetItem in ipairs(TSMAPI:GetConversionTargetItems("mill")) do
		local herbs = TSMAPI:GetItemConversions(targetItem)
		if herbs[itemString] then
			local matValue = TSMAPI:GetItemValue(targetItem, "DBMarket")
			value = value + (matValue or 0) * herbs[itemString].rate
		end
	end
	
	value = floor(value)
	return value > 0 and value or nil
end

function ZWM:GetProspectValue(itemString)
	local value = 0
	
	for _, targetItem in ipairs(TSMAPI:GetConversionTargetItems("prospect")) do
		local gems = TSMAPI:GetItemConversions(targetItem)
		if gems[itemString] then
			local matValue = TSMAPI:GetItemValue(targetItem, "DBMarket")
			value = value + (matValue or 0) * (gems[itemString].rate /5)
		end
	end
	
	value = floor(value)
	return value > 0 and value or nil
end

function ZWM:LoadTooltipOptions(container)
  local page = {
    {
      type = "SimpleGroup",
      layout = "Flow",
      fullHeight = true,
      children = {
        {
          type = "CheckBox",
          label = L["Enable display of Zotwee data in tooltip."],
          relativeWidth = 1,
          settingInfo = { ZWM.db.profile, "tooltip" },
          callback = function(_, _, value)
            container:ReloadTab()
          end,
        },
        {
          type = "CheckBox",
          label = L["Display realm market price in tooltip."],
          disabled = not ZWM.db.profile.tooltip,
          relativeWidth = 0.49,
          settingInfo = { ZWM.db.profile, "marketValue" },
          tooltip = L["If checked, the market value of the item will be shown"],
        },
        {
          type = "CheckBox",
          label = L["Display last 24 hours realm price in tooltip."],
          disabled = not ZWM.db.profile.tooltip,
          settingInfo = { ZWM.db.profile, "last24Price" },
          tooltip = L["If checked, the last 24 hours price of the item will be shown."],
        },
        {
          type = "CheckBox",
          label = L["Display last 30 days realm price in tooltip."],
          disabled = not ZWM.db.profile.tooltip,
          settingInfo = { ZWM.db.profile, "last30Price" },
          tooltip = L["If checked, the last 30 days price of the item will be shown."],
        },
        {
          type = "CheckBox",
          label = L["Display minimum buyout realm price in tooltip."],
          disabled = not ZWM.db.profile.tooltip,
          settingInfo = { ZWM.db.profile, "minBuyout" },
          tooltip = L["If checked, the minimum buyout price of the item will be shown."],
        },
        {
          type = "CheckBox",
          label = L["Display the lastseen time for the item in tooltip."],
          disabled = not ZWM.db.profile.tooltip,
          settingInfo = { ZWM.db.profile, "lastSeen" },
          tooltip = L["If checked, the lastseen time for the item will be shown."],
        },
        {
          type = "CheckBox",
          label = L["Display the lastseen quantity for the item in tooltip."],
          disabled = not ZWM.db.profile.tooltip,
          settingInfo = { ZWM.db.profile, "quantity" },
          tooltip = L["If checked, the lastseen quantity for the item will be shown."],
        },
        {
          type = "CheckBox",
          label = L["Suggests the item action to be taken based on prices."],
          disabled = not ZWM.db.profile.tooltip,
          settingInfo = { ZWM.db.profile, "suggestion" },
          tooltip = L["If checked, the suggestion will be shown."],
        },
        {
          type = "Dropdown",
          label = L["Price Source to be used for suggestions."],
          list = { "DBMarket", "zotweeMarket" },
          settingInfo = { ZWM.db.profile, "suggestionSource" },
          relativeWidth = 0.5,
          tooltip = L["Price Source to be used for suggestions."],
        },
      },
    },
  }

  TSMAPI:BuildPage(container, page)
end
